@@ -47,6 +47,7 @@ AutoCorrection::AutoCorrection()
       mMaxFindStringLenght(0),
       mMinFindStringLenght(0)
 {
+    mNonBreakingSpace = QChar(QChar::Nbsp);
     // default double quote open 0x201c
     // default double quote close 0x201d
     // default single quote open 0x2018
@@ -80,9 +81,7 @@ void AutoCorrection::selectStringOnMaximumSearchString(QTextCursor &cursor, int
     //TODO if not => check if pos -1 is a space => not a piece of word
     //TODO otherwise move cursor until we detect a space
     //TODO otherwise we must not autoconvert it.
-    if (pos == block.position()) {
-        //it's ok
-    } else {
+    if (pos != block.position()) {
         const QString text = block.text();
         const int currentPos = (pos - block.position());
         if (!text.at(currentPos - 1).isSpace()) {
@@ -179,7 +178,9 @@ bool AutoCorrection::autocorrect(bool htmlMode, QTextDocument &document, int &po
             fixTwoUppercaseChars();
             capitalizeWeekDays();
             replaceTypographicQuotes();
-            addNonBreakingSpace();
+            if (mWord.length() <= 2) {
+                addNonBreakingSpace();
+            }
         }
 
         if (mCursor.selectedText() != mWord) {
@@ -188,7 +189,7 @@ bool AutoCorrection::autocorrect(bool htmlMode, QTextDocument &document, int &po
         position = oldPosition;
 
         if (!done) {
-            selectStringOnMaximumSearchString(mCursor, oldPosition);
+            selectStringOnMaximumSearchString(mCursor, position);
             mWord = mCursor.selectedText();
             if (!mWord.isEmpty()) {
                 const int newPos = advancedAutocorrect();
@@ -417,8 +418,11 @@ void AutoCorrection::superscriptAppendix()
                 QString::ConstIterator constIter = number.constBegin();
                 bool found = true;
                 // don't apply superscript to 1th, 2th and 3th
-                if (number.length() == 1 &&
-                        (*constIter == QLatin1Char('1') || *constIter == QLatin1Char('2') || *constIter == QLatin1Char('3'))) {
+                const int numberLength(number.length());
+                if (numberLength == 1 &&
+                        (*constIter == QLatin1Char('1') ||
+                         *constIter == QLatin1Char('2') ||
+                         *constIter == QLatin1Char('3'))) {
                     found = false;
                 }
                 if (found) {
@@ -430,7 +434,7 @@ void AutoCorrection::superscriptAppendix()
                         ++constIter;
                     }
                 }
-                if (found && number.length() + i.value().length() == trimmedLenght) {
+                if (found && numberLength + i.value().length() == trimmedLenght) {
                     startPos = mCursor.selectionStart() + pos;
                     endPos = startPos - pos + trimmedLenght;
                     break;
@@ -454,7 +458,42 @@ void AutoCorrection::superscriptAppendix()
 void AutoCorrection::addNonBreakingSpace()
 {
     if (mAddNonBreakingSpace && isFrenchLanguage()) {
-        if (mWord.length() == 1) {
+        const QTextBlock block = mCursor.block();
+        const QString text = block.text();
+        const QChar lastChar = text.at(mCursor.position() - 1 - block.position());
+
+        if (lastChar == QLatin1Char(':') ||
+                lastChar == QLatin1Char(';') ||
+                lastChar == QLatin1Char('!') ||
+                lastChar == QLatin1Char('?') ||
+                lastChar == QLatin1Char('%')) {
+            const int pos = mCursor.position() - 2 - block.position();
+            if (pos >= 0) {
+                const QChar previousChar = text.at(pos);
+                if (previousChar.isSpace()) {
+                    QTextCursor cursor(mCursor);
+                    cursor.setPosition(pos);
+                    cursor.setPosition(pos + 1, QTextCursor::KeepAnchor);
+                    cursor.deleteChar();
+                    mCursor.insertText(mNonBreakingSpace);
+                }
+            }
+        } else {
+            //°C (degres)
+            const QChar previousChar = text.at(mCursor.position() - 2 - block.position());
+            if (lastChar == QLatin1Char('C') && previousChar == QChar(0x000B0)) {
+                const int pos = mCursor.position() - 3 - block.position();
+                if (pos >= 0) {
+                    const QChar previousChar = text.at(pos);
+                    if (previousChar.isSpace()) {
+                        QTextCursor cursor(mCursor);
+                        cursor.setPosition(pos);
+                        cursor.setPosition(pos + 1, QTextCursor::KeepAnchor);
+                        cursor.deleteChar();
+                        mCursor.insertText(mNonBreakingSpace);
+                    }
+                }
+            }
         }
     }
 }
@@ -716,6 +755,11 @@ bool AutoCorrection::excludeToUppercase(const QString &word) const
     return false;
 }
 
+void AutoCorrection::setNonBreakingSpace(const QChar &nonBreakingSpace)
+{
+    mNonBreakingSpace = nonBreakingSpace;
+}
+
 void AutoCorrection::uppercaseFirstCharOfSentence()
 {
     if (!mUppercaseFirstCharOfSentence) {
@@ -815,16 +859,13 @@ int AutoCorrection::advancedAutocorrect()
         return -1;
     }
 
-    const int startPos = mCursor.selectionStart();
-    const int length = mWord.length();
-
     const QString trimmedWord = mWord.trimmed();
-    QString actualWord = trimmedWord;
-
-    if (actualWord.isEmpty()) {
+    if (trimmedWord.isEmpty()) {
         return -1;
     }
 
+    QString actualWord = trimmedWord;
+
     const int actualWordLength(actualWord.length());
     if (actualWordLength < mMinFindStringLenght) {
         return -1;
@@ -833,6 +874,8 @@ int AutoCorrection::advancedAutocorrect()
         return -1;
     }
 
+    const int startPos = mCursor.selectionStart();
+    const int length = mWord.length();
     // If the last char is punctuation, drop it for now
     bool hasPunctuation = false;
     const QChar lastChar = actualWord.at(actualWord.length() - 1);
@@ -918,7 +961,6 @@ void AutoCorrection::replaceTypographicQuotes()
     //     b. and the previous quote of a different kind (so that we get empty quotations right)
 
     bool ending = true;
-    const QChar nbsp = QChar(QChar::Nbsp);
     for (int i = mWord.length(); i > 1; --i) {
         const QChar c = mWord.at(i - 1);
         if (c == QLatin1Char('"') || c == QLatin1Char('\'')) {
@@ -955,31 +997,31 @@ void AutoCorrection::replaceTypographicQuotes()
             }
 
             if (doubleQuotes && mReplaceDoubleQuotes) {
-                if (!ending) {
+                if (ending) {
                     if (addNonBreakingSpace) {
-                        mWord.replace(i - 1, 2, QString(nbsp + mTypographicDoubleQuotes.begin));
-                    } else {
-                        mWord[i - 1] = mTypographicDoubleQuotes.begin;
-                    }
-                } else {
-                    if (addNonBreakingSpace) {
-                        mWord.replace(i - 1, 2, QString(nbsp + mTypographicDoubleQuotes.end));
+                        mWord.replace(i - 1, 2, QString(mNonBreakingSpace + mTypographicDoubleQuotes.end));
                     } else {
                         mWord[i - 1] = mTypographicDoubleQuotes.end;
                     }
+                } else {
+                    if (addNonBreakingSpace) {
+                        mWord.replace(i - 1, 2, QString(mNonBreakingSpace + mTypographicDoubleQuotes.begin));
+                    } else {
+                        mWord[i - 1] = mTypographicDoubleQuotes.begin;
+                    }
                 }
             } else if (mReplaceSingleQuotes) {
-                if (!ending) {
+                if (ending) {
                     if (addNonBreakingSpace) {
-                        mWord.replace(i - 1, 2, QString(nbsp + mTypographicSingleQuotes.begin));
+                        mWord.replace(i - 1, 2, QString(mNonBreakingSpace + mTypographicSingleQuotes.end));
                     } else {
-                        mWord[i - 1] = mTypographicSingleQuotes.begin;
+                        mWord[i - 1] = mTypographicSingleQuotes.end;
                     }
                 } else {
                     if (addNonBreakingSpace) {
-                        mWord.replace(i - 1, 2, QString(nbsp + mTypographicSingleQuotes.end));
+                        mWord.replace(i - 1, 2, QString(mNonBreakingSpace + mTypographicSingleQuotes.begin));
                     } else {
-                        mWord[i - 1] = mTypographicSingleQuotes.end;
+                        mWord[i - 1] = mTypographicSingleQuotes.begin;
                     }
                 }
             }
@@ -988,16 +1030,14 @@ void AutoCorrection::replaceTypographicQuotes()
 
     // first character
     if (mWord.at(0) == QLatin1Char('"') && mReplaceDoubleQuotes) {
+        mWord[0] = mTypographicDoubleQuotes.begin;
         if (addNonBreakingSpace) {
-            mWord.replace(0, 2, QString(mTypographicDoubleQuotes.begin + nbsp));
-        } else {
-            mWord[0] = mTypographicDoubleQuotes.begin;
+            mWord.insert(1, mNonBreakingSpace);
         }
     } else if (mWord.at(0) == QLatin1Char('\'') && mReplaceSingleQuotes) {
+        mWord[0] = mTypographicSingleQuotes.begin;
         if (addNonBreakingSpace) {
-            mWord.replace(0, 2, QString(mTypographicSingleQuotes.begin + nbsp));
-        } else {
-            mWord[0] = mTypographicSingleQuotes.begin;
+            mWord.insert(1, mNonBreakingSpace);
         }
     }
 }
@@ -1005,8 +1045,9 @@ void AutoCorrection::replaceTypographicQuotes()
 void AutoCorrection::readAutoCorrectionXmlFile(bool forceGlobal)
 {
     auto kdelang = QLocale::system().uiLanguages().at(0);
-    if (kdelang == QLatin1String("C"))
+    if (kdelang == QLatin1String("C")) {
         kdelang = QStringLiteral("en_US");
+    }
     kdelang.remove(QRegularExpression(QStringLiteral("@.*")));
 
     mUpperCaseExceptions.clear();
